/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 2001-2007, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

------------------------------------------------------------------------------

toupper.c is for converting all characters in a text file to upper
case.

John Conover
January 4, 2005
john@email.johncon.com

$Revision: 0.0 $
$Date: 2005/01/04 19:36:00 $
$Id: toupper.c,v 0.0 2005/01/04 19:36:00 john Exp $
$Log: toupper.c,v $
Revision 0.0  2005/01/04 19:36:00  john
Initial version


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> /* for getopt() */
#include <ctype.h> /* for toupper(), tolower() */

static char rcsid[] = "$Id: toupper.c,v 0.0 2005/01/04 19:36:00 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 2001-2007, John Conover, All Rights Reserved"; /* the copyright banner */

#define BUFLEN BUFSIZ /* i/o buffer size */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "Convert all characters in a text file to upper case\n",
    "Usage: toupper [-v] [filename(s)]\n",
    "    -v, print the program's version information\n",
    "    filename(s), filename(s), (defaults to stdin)\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
    "Error opening file\n",
    "Error closing file\n"
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1
#define EOPEN 2
#define ECLOSE 3

#ifdef __STDC__

static void print_message (int retval); /* print any error messages */

#else

static void print_message (); /* print any error messages */

#endif

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    char buffer[BUFLEN], /* i/o buffer */
         *temp; /* reference to characters in i/o buffer */

    int retval = NOERROR, /* return value, assume no error */
        c; /* command line switch */

    FILE *infile = stdin; /* reference to stdin */

    while ((c = getopt (argc, argv, "v")) != EOF) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'v': /* request for program's version information? */

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (retval == NOERROR) /* any errors? */
    {

        do /* while there are input file(s), if any, listed on the command line */
        {
            retval = EOPEN; /* assume error opening file */

            if ((infile = argc <= optind ? stdin : fopen (argv[optind], "r")) != (FILE *) 0) /* open the input file; if none specified, default to stdin */
            {
                retval = NOERROR; /* assume no error */

                while ((fgets (buffer, BUFLEN, infile) != (char *) 0)) /* read the records from the input file until EOF, or an error in processing, or a null line, (e.g., start of message body) */
                {
                    temp = buffer; /* reference the first character in the record */

                    while (*temp != '\0') /* for each character in the record */
                    {
                        *temp = (char) toupper (*temp); /* convert the character to uppercase */
                        temp ++; /* next character */
                    }

                    (void) printf ("%s", buffer); /* print the converted record to stdout */
                }

                if (argc > optind) /* using stdin as input? */
                {

                    if (fclose (infile) == EOF) /* no, close the input file */
                    {
                        retval = ECLOSE; /* error closing file */
                    }

                }

            }

        }
        while ((argc > ++ optind) && (retval == NOERROR)); /* while there are any remaining input file(s), if any, listed on the command line */
    }

    print_message (retval); /* print any error messages */
    exit (retval); /* exit with the error value */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}
